from h5py import File, Group
from thd_json import thd_joint_states
from h5schemas.scan_schema import open_schema
from h5schemas.initialize_dataset import initialize_dataset
from h5schemas.joint.add_joint import add_joint_sample
from h5schemas.header.add_header import add_header_sample, check_uuid_time_stamp


def init_thd_joint_states(h5_file: File | Group):
    schema_groups = get_source_schemas()
    joints = h5_file.require_group("joints")

    initialize_dataset(joints, schema_groups)


def get_source_schemas():
    return open_schema(thd_joint_states, "thd_joint_states.json")


def add_thd_joint_states_sample(
    h5_file: File | Group,
    master_joint_e1: float | None = None,
    master_joint_a1: float | None = None,
    master_joint_a2: float | None = None,
    master_joint_a3: float | None = None,
    master_joint_a4: float | None = None,
    master_joint_a5: float | None = None,
    master_joint_a6: float | None = None,
    slave_joint_e1: float | None = None,
    slave_joint_a1: float | None = None,
    slave_joint_a2: float | None = None,
    slave_joint_a3: float | None = None,
    slave_joint_a4: float | None = None,
    slave_joint_a5: float | None = None,
    slave_joint_a6: float | None = None,
    table_joint_a1: float | None = None,
    uuid: str | None = None,
    timestamp: int | None = None,
):
    uuid, timestamp = check_uuid_time_stamp(uuid, timestamp)

    add_joint_sample(h5_file["./joints/master_joint_e1"], master_joint_e1)

    add_joint_sample(h5_file["./joints/master_joint_a1"], master_joint_a1)
    add_joint_sample(h5_file["./joints/master_joint_a2"], master_joint_a2)
    add_joint_sample(h5_file["./joints/master_joint_a3"], master_joint_a3)
    add_joint_sample(h5_file["./joints/master_joint_a4"], master_joint_a4)
    add_joint_sample(h5_file["./joints/master_joint_a5"], master_joint_a5)
    add_joint_sample(h5_file["./joints/master_joint_a6"], master_joint_a6)

    add_joint_sample(h5_file["./joints/slave_joint_e1"], slave_joint_e1)

    add_joint_sample(h5_file["./joints/slave_joint_a1"], slave_joint_a1)
    add_joint_sample(h5_file["./joints/slave_joint_a2"], slave_joint_a2)
    add_joint_sample(h5_file["./joints/slave_joint_a3"], slave_joint_a3)
    add_joint_sample(h5_file["./joints/slave_joint_a4"], slave_joint_a4)
    add_joint_sample(h5_file["./joints/slave_joint_a5"], slave_joint_a5)
    add_joint_sample(h5_file["./joints/slave_joint_a6"], slave_joint_a6)

    add_joint_sample(h5_file["./joints/table_joint_a1"], table_joint_a1)

    add_header_sample(h5_file["./joints/header"], uuid, timestamp)


if __name__ == "__main__":
    import numpy as np

    test_file = File("./examples/thd_joint_joint.h5", "w")
    init_thd_joint_states(test_file)

    add_thd_joint_states_sample(test_file, *np.arange(15))
    add_thd_joint_states_sample(test_file, *np.arange(15, 30))
